@@ -24,6 +24,8 @@
 *
 */
 
+#include <kdb/kdb-priv.h> /* KDBManagerCheckAd */
+
 #define TRACK_REFERENCES 0
 
 #define KONST const
@@ -41,6 +43,8 @@
 #include <klib/printf.h>
 #include <klib/rc.h>
 
+#include <vfs/path.h> /* VFSManagerMakePath */
+
 #include <kfs/arc.h>
 #include <os-native.h>
 #include <sysalloc.h>
@@ -207,40 +211,14 @@ rc_t KTableMake ( KTable **tblp, const KDirectory *dir, const char *path )
     KRefcountInit ( & tbl -> refcount, 1, "KTable", "make", path );
     strcpy ( tbl -> path, path );
 
+    /* YES,
+      DBG_VFS should be used here to be printed along with other VFS messages */
+    DBGMSG(DBG_VFS, DBG_FLAG(DBG_VFS_SERVICE), ("Making KTable '%s'\n", path));
+     
     * tblp = tbl;
     return 0;
 }
 
-static void ad(const KDBManager * self, const char * aPath, char ** path)
-{
-    const char *slash = strrchr(aPath, '/');
-
-    assert(self && aPath && path);
-
-    if (slash != NULL)
-        ++slash;
-    else
-        slash = aPath;
-
-    if ((KDirectoryPathType(self->wd, aPath) & ~kptAlias) != kptDir)
-        return;
-
-    if ((KDirectoryPathType(self->wd, "%s/%s.sra", aPath, slash)
-        & ~kptAlias) != kptFile)
-    {
-        return;
-    }
-
-    *path = calloc(1, strlen(aPath) + 6 + strlen(slash));
-
-    if (*path != NULL)
-        sprintf(*path, "%s/%s.sra", aPath, slash);
-
-    /* YES, DBG_VFS should be used here to be printed along with other VFS messages */
-    DBGMSG(DBG_VFS, DBG_FLAG(DBG_VFS_SERVICE),
-        ("'%s' was sesolved to '%s'\n", aPath, *path ));
-}
-
 /* OpenTableRead
  * VOpenTableRead
  *  open a table for read
@@ -256,10 +234,14 @@ rc_t KDBManagerVOpenTableReadInt ( const KDBManager *self,
     const char *path, va_list args, const struct VPath *vpath )
 {
     rc_t rc;
-    char aTblpath[4096];
+    char aTblpath[4096] = "";
     char * tblpath = aTblpath;
-    int z = ( args == NULL ) ?
-        snprintf ( aTblpath, sizeof aTblpath, "%s", path ):
+    int z = 0;
+    if (args == NULL) {
+        if (path != NULL)
+            z = snprintf(aTblpath, sizeof aTblpath, "%s", path);
+    }
+    else
         vsnprintf ( aTblpath, sizeof aTblpath, path, args );
     if ( z < 0 || ( size_t ) z >= sizeof aTblpath )
         rc = RC ( rcDB, rcMgr, rcOpening, rcPath, rcExcessive );
@@ -268,34 +250,91 @@ rc_t KDBManagerVOpenTableReadInt ( const KDBManager *self,
         KTable *tbl;
         const KDirectory *dir;
         bool prerelease = false;
+        const VPath *path2 = NULL;
 
-        ad(self, aTblpath, &tblpath);
+        {
+            rc_t rc = 0;
+            VPath *path = NULL;
+            if (vpath == NULL)
+                rc = VFSManagerMakePath(self->vfsmgr, &path, "%s", aTblpath);
+            if (rc == 0) {
+                const String * str = NULL;
+                KDBManagerCheckAd(self, path != NULL ? path : vpath, &path2);
+                if (path2 != NULL) {
+                    rc = VPathMakeString(path2, &str);
+                    if (rc == 0) {
+                        assert(str);
+                        tblpath = calloc(1, str->size + 1);
+                        if (tblpath != NULL)
+                            string_printf(tblpath, str->size + 1, NULL,
+                                "%S", str);
+                        StringWhack(str);
+                    }
+                }
+                else {
+                    rc = VPathMakeString(path != NULL ? path : vpath, &str);
+                    if (rc == 0) {
+                        assert(str);
+                        tblpath = calloc(1, str->size + 1);
+                        if (tblpath != NULL)
+                            string_printf(tblpath, str->size + 1, NULL,
+                                "%S", str);
+                        StringWhack(str);
+                    }
+                }
+                VPathRelease(path);
+            }
+        }
 
         rc = KDBOpenPathTypeRead ( self, wd, tblpath, &dir, kptTable, NULL,
-            try_srapath, vpath );
+            try_srapath, path2 != NULL ? path2 : vpath );
         if ( rc != 0 )
         {
             prerelease = true;
-            rc = KDBOpenPathTypeRead ( self, wd, tblpath, &dir, kptPrereleaseTbl, NULL,
-                try_srapath, vpath );
+            rc = KDBOpenPathTypeRead ( self, wd, tblpath, &dir,
+                kptPrereleaseTbl, NULL,
+                try_srapath, path2 != NULL ? path2 : vpath);
         }
 
-        if (aTblpath != tblpath)
-            free(tblpath);
-
-        if ( rc == 0 )
+        if (rc == 0)
         {
-            rc = KTableMake ( & tbl, dir, tblpath );
+            String str;
+            const char * p = tblpath;
+            if (p == NULL) {
+                if (path2 != NULL) {
+                    rc_t rc = VPathGetPath(vpath, &str);
+                    if (rc == 0)
+                        p = str.addr;
+                }
+                else if (vpath != NULL) {
+                    rc_t rc = VPathGetPath(vpath, &str);
+                    if (rc == 0)
+                        p = str.addr;
+                }
+            }
+
+            VPathRelease(path2);
+            path2 = NULL;
+
+            rc = KTableMake ( & tbl, dir, p );
             if ( rc == 0 )
             {
                 tbl -> mgr = KDBManagerAttach ( self );
                 tbl -> prerelease = prerelease;
                 * tblp = tbl;
+
+                if (aTblpath != tblpath)
+                    free(tblpath);
+
                 return 0;
             }
             KDirectoryRelease ( dir );
         }
     }
+
+    if (aTblpath != tblpath)
+        free(tblpath);
+
     return rc;
 }
 
