@@ -23,11 +23,12 @@
 * ==============================================================================
 *
 */
+
 #include <kns/extern.h>
 
 #include "http-file-priv.h"
 
-#include "http-priv.h"
+#include "http-priv.h" /* SUPPORT_CHUNKED_READ */
 #include "mgr-priv.h"
 #include "stream-priv.h"
 
@@ -68,8 +69,6 @@
 #include <ctype.h>
 #include <assert.h>
 
-#define SUPPORT_CHUNKED_READ 1
-
 #if SUPPORT_CHUNKED_READ
 #include <kfs/chunk-reader.h>
 #endif
@@ -86,6 +85,10 @@
 #define USE_CACHE_CONTROL 1
 #define NO_CACHE_LIMIT ( ( uint64_t ) ( 16 * 1024 * 1024 ) )
 
+/* VDB-4146: Use Content-Length from HEAD in GET
+   when GET doesn't return Content-Length and Range was not requested. */
+#define USE_SIZE_FROM_HEAD
+
 static
 rc_t CC KHttpFileDestroy ( KHttpFile *self )
 {
@@ -187,6 +190,8 @@ rc_t KHttpFileMakeRequest ( const KHttpFile *cself, uint64_t pos, size_t req_siz
                     TRACE ( "KClientHttpRequestByteRange ( req, pos=%lu, bsize=%lu ); failed: rc=%u\n",
                             pos, req_size, rc );
                 }
+                else
+                    req->rangeRequested = true;
             }
 
             if ( rc == 0 )
@@ -238,6 +243,8 @@ rc_t KHttpFileMakeRequest ( const KHttpFile *cself, uint64_t pos, size_t req_siz
             }
         }
 
+        if (rc == 0 && rslt != NULL && *rslt != NULL)
+            (*rslt)->rangeRequested = req->rangeRequested;
         KClientHttpRequestRelease ( req );
     }
     return rc;
@@ -309,6 +316,12 @@ rc_t KHttpFileTimedReadInt ( const KHttpFile * self,
 
                 /* extract stated bytes returned - must be whole file */
                 have_size = KClientHttpResultSize ( rslt, & result_size64 );
+#ifdef USE_SIZE_FROM_HEAD
+                if (!have_size && !rslt->rangeRequested) {
+                    result_size64 = self->file_size;
+                    have_size = true;
+                }
+#endif
                 result_size = ( size_t ) result_size64;
                 if ( pos != 0 || ! have_size || result_size > bsize )
                 {
@@ -958,6 +971,12 @@ rc_t KHttpFileTimedReadChunkedInt ( const KHttpFile * self,
 
                 /* extract stated bytes returned - must be whole file */
                 have_size = KClientHttpResultSize ( rslt, & result_size64 );
+#ifdef USE_SIZE_FROM_HEAD
+                if (!have_size && !rslt->rangeRequested) {
+                    result_size64 = self->file_size;
+                    have_size = true;
+                }
+#endif
                 result_size = ( size_t ) result_size64;
                 if ( pos != 0 || ! have_size || result_size > bytes )
                 {
@@ -1229,7 +1248,8 @@ static KFile_vt_v1 vtKHttpFile =
 };
 
 static
-rc_t KHttpFileMake( KHttpFile ** self, const char *url, va_list args )
+rc_t KHttpFileMake( KHttpFile ** self,
+             const KDataBuffer * aBuf, const char *url, va_list args )
 {
     rc_t rc;
     KHttpFile * f = calloc ( 1, sizeof *f );
@@ -1249,7 +1269,12 @@ rc_t KHttpFileMake( KHttpFile ** self, const char *url, va_list args )
                 rc = KDataBufferMake( buf, 8, 0 );
                 if ( rc == 0 )
                 {
-                    rc = KDataBufferVPrintf ( buf, url, args );
+                    if (aBuf != NULL)
+                        rc = KDataBufferPrintf(buf,
+                          "%.*s", aBuf->elem_count, aBuf->base);
+                    else
+                        rc = KDataBufferVPrintf ( buf, url, args );
+
                     if ( rc == 0 )
                     {
                         rc = ParseUrl ( & f -> block, buf -> base, buf -> elem_count - 1 );
@@ -1266,11 +1291,12 @@ rc_t KHttpFileMake( KHttpFile ** self, const char *url, va_list args )
         }
         free ( f );
     }
+
     return rc;
 }
 
-static rc_t KNSManagerVMakeHttpFileInt ( const KNSManager *self,
-    const KFile **file, KStream *conn, ver_t vers, bool reliable, bool need_env_token, bool payRequired,
+static rc_t KNSManagerVMakeHttpFileIntUnstableImpl( const KNSManager *self,
+    const KFile **file, KStream *conn, ver_t vers, bool reliable, bool need_env_token, bool payRequired, const KDataBuffer *buf,
     const char *url, va_list args )
 {
     rc_t rc;
@@ -1288,7 +1314,8 @@ static rc_t KNSManagerVMakeHttpFileInt ( const KNSManager *self,
         else
         {
             KHttpFile * f;
-            rc = KHttpFileMake ( &f, url, args );
+            rc = KHttpFileMake ( &f, buf, url, args );
+
             if ( rc == 0 )
             {
                 KDataBuffer * buf = & f -> orig_url_buffer;
@@ -1432,8 +1459,8 @@ static rc_t KNSManagerVMakeHttpFileInt ( const KNSManager *self,
                                         if ( ! reliable )
                                             print = false;
                                         if ( print ) {
-                                        assert ( buf );
-                                        PLOGERR ( klogErr,
+                                           assert ( buf );
+                                           PLOGERR ( klogErr,
                                             ( klogErr, rc,
                                             "Failed to KNSManagerVMakeHttpFileInt('$(path)' ($(ip)))"
                                             " from '$(local)'", "path=%.*s,ip=%s,local=%s",
@@ -1465,29 +1492,20 @@ static rc_t KNSManagerVMakeHttpFileInt ( const KNSManager *self,
 
 /******************************************************************************/
 
-LIB_EXPORT rc_t CC KNSManagerMakeHttpFile(const KNSManager *self,
-    const KFile **file, struct KStream *conn, ver_t vers, const char *url, ...)
+rc_t KNSManagerVMakeHttpFileIntUnstableFromBuffer(const KNSManager *self,
+    const KFile **file, KStream *conn, ver_t vers, bool reliable,
+    bool need_env_token, bool payRequired, const char *url,
+    const KDataBuffer *buf)
 {
-    rc_t rc = 0;
-    va_list args;
-    va_start(args, url);
-    rc = KNSManagerVMakeHttpFileInt ( self, file, conn, vers, false, false, false, url, args);
-    va_end(args);
-    return rc;
+    return KNSManagerVMakeHttpFileIntUnstableImpl(self, file, conn, vers,
+        reliable, need_env_token, payRequired, buf, url, NULL);
 }
 
-LIB_EXPORT rc_t CC KNSManagerMakeReliableHttpFile(const KNSManager *self,
-    const KFile **file, struct KStream *conn, ver_t vers, bool reliable, bool need_env_token, bool payRequired, const char *url, ...)
+rc_t KNSManagerVMakeHttpFileIntUnstable(const KNSManager *self,
+    const KFile **file, KStream *conn, ver_t vers, bool reliable,
+    bool need_env_token, bool payRequired,
+    const char *url, va_list args)
 {
-    rc_t rc = 0;
-    va_list args;
-    va_start(args, url);
-    rc = KNSManagerVMakeHttpFileInt ( self, file, conn, vers, true, need_env_token, payRequired, url, args);
-    va_end(args);
-    return rc;
-}
-
-LIB_EXPORT bool CC KFileIsKHttpFile ( const struct KFile * self )
-{
-    return self != NULL && &self->vt->v1 == &vtKHttpFile;
+    return KNSManagerVMakeHttpFileIntUnstableImpl(self, file, conn, vers,
+        reliable, need_env_token, payRequired, NULL, url, args);
 }
