@@ -26,12 +26,13 @@
 
 #include <kns/extern.h>
 
-#include <klib/rc.h>
-#include <klib/data-buffer.h>
-#include <klib/printf.h>
-#include <klib/log.h>
 #include <klib/base64.h>
+#include <klib/data-buffer.h>
 #include <klib/debug.h>
+#include <klib/log.h>
+#include <klib/printf.h>
+#include <klib/rc.h>
+#include <klib/sra-release-version.h> /* SraReleaseVersion */
 
 #include <cloud/manager.h>
 
@@ -145,7 +146,8 @@ rc_t KClientHttpMakeRequestInt ( const KClientHttp *self,
     rc_t rc;
 
     /* create the object with empty buffer */
-    KClientHttpRequest * req = calloc ( 1, sizeof * req );
+    KClientHttpRequest * req
+        = (KClientHttpRequest *) calloc ( 1, sizeof * req );
     if ( req == NULL )
         rc = RC ( rcNS, rcNoTarg, rcAllocating, rcMemory, rcNull );
     else
@@ -160,7 +162,8 @@ rc_t KClientHttpMakeRequestInt ( const KClientHttp *self,
             rc = KDataBufferMakeBytes ( & req -> body, 0 );
             if ( rc == 0 )
             {
-                KRefcountInit ( & req -> refcount, 1, "KClientHttpRequest", "make", buf -> base );
+                KRefcountInit ( & req -> refcount, 1, "KClientHttpRequest",
+                    "make", (char*) buf -> base );
 
                 /* fill out url_buffer with URL */
                 rc = KClientHttpRequestInit ( req, block, buf );
@@ -226,7 +229,8 @@ LIB_EXPORT rc_t CC KClientHttpVMakeRequest ( const KClientHttp *self,
                 {
                     /* parse the URL */
                     URLBlock block;
-                    rc = ParseUrl ( & block, buf . base, buf . elem_count - 1 );
+                    rc = ParseUrl ( & block, (char*) buf . base,
+                        buf . elem_count - 1 );
                     if ( rc == 0 )
                         rc = KClientHttpMakeRequestInt ( self, _req, & block, & buf );
                 }
@@ -303,7 +307,8 @@ rc_t CC KNSManagerMakeClientRequestInt ( const KNSManager *self,
                 {
                     /* parse the URL */
                     URLBlock block;
-                    rc = ParseUrl ( & block, buf . base, buf . elem_count - 1 );
+                    rc = ParseUrl ( & block, (char*) buf . base,
+                        buf . elem_count - 1 );
                     if ( rc == 0 )
                     {
                         KClientHttp * http;
@@ -731,7 +736,7 @@ rc_t
 UrlEncode( const char * source, size_t size, char ** res )
 {   /* source: https://www.tutorialspoint.com/html/html_url_encoding.htm */
     char * cur;
-    int i;
+    size_t i = 0;
     assert ( source != NULL );
     assert ( res != NULL );
 
@@ -837,7 +842,9 @@ LIB_EXPORT rc_t CC KClientHttpRequestVAddQueryParam ( KClientHttpRequest *self,
                             KDataBufferWhack( & self -> url_buffer );
                             self -> url_buffer = newBuf;
                             /* re-parse the new URL */
-                            rc = ParseUrl ( & self -> url_block, self -> url_buffer . base, self -> url_buffer . elem_count - 1 );
+                            rc = ParseUrl ( & self -> url_block,
+                                (char*) self -> url_buffer . base,
+                                self -> url_buffer . elem_count - 1 );
                         }
                     }
                     free ( encValue );
@@ -862,6 +869,58 @@ LIB_EXPORT rc_t CC KClientHttpRequestAddQueryParam ( KClientHttpRequest *self, c
     return rc;
 }
 
+rc_t KClientHttpRequestUrlEncodeBase64(const String ** encoding) {
+    int n = 0;
+    size_t i = 0;
+
+    if (encoding == NULL || *encoding == NULL || (*encoding)->addr == NULL)
+        return 0;
+
+    for (i = 0; i < (*encoding)->size; ++i) {
+        if (((*encoding)->addr)[i] == '+' ||
+            ((*encoding)->addr)[i] == '/')
+        {
+            ++n;
+        }
+    }
+
+    if (n > 0) {
+        size_t iFrom = 0, iTo = 0;
+        const char *from = (*encoding)->addr;
+        char *to = NULL;
+        uint32_t len = (*encoding)->size + n + n;
+
+        String * encoded = (String *) calloc(1, sizeof * encoded + len + 1);
+        if (encoded == NULL)
+            return RC(rcNS, rcString, rcAllocating, rcMemory, rcExhausted);
+
+        to = (char*)(encoded + 1);
+        StringInit(encoded, to, len, len);
+
+        for (iFrom = 0; iFrom < (*encoding)->size; ++iFrom) {
+            if (from[iFrom] == '+') {
+                to[iTo++] = '%';
+                to[iTo++] = '2';
+                to[iTo++] = 'b';
+            }
+            else if (from[iFrom] == '/') {
+                to[iTo++] = '%';
+                to[iTo++] = '2';
+                to[iTo++] = 'f';
+            }
+            else
+                to[iTo++] = from[iFrom];
+        }
+        to[iTo] = '\0';
+        assert(iTo == len);
+
+        StringWhack(*encoding);
+        *encoding = encoded;
+    }
+
+    return 0;
+}
+
 LIB_EXPORT rc_t CC KClientHttpRequestAddPostFileParam ( KClientHttpRequest * self, const char * name, const char * path )
 {
     rc_t rc = 0;
@@ -902,8 +961,10 @@ LIB_EXPORT rc_t CC KClientHttpRequestAddPostFileParam ( KClientHttpRequest * sel
                             rc = KMMapAddrRead( mm, & fileStart );
                             if ( rc == 0 )
                             {
-                                const String * encoded;
+                                const String * encoded = NULL;
                                 rc = encodeBase64( & encoded, fileStart, fileSize );
+                                if ( rc == 0 )
+                                    rc = KClientHttpRequestUrlEncodeBase64( & encoded );
                                 if ( rc == 0 )
                                 {
                                     rc = KClientHttpRequestAddPostParam( self, "%s=%S", name, encoded );
@@ -1167,10 +1228,17 @@ rc_t CC KClientHttpRequestFormatMsgInt( const KClientHttpRequest *self,
 {
     rc_t rc;
     rc_t r2 = 0;
-    bool have_user_agent = false;
+
     bool have_accept = false;
-    String user_agent_string;
+    bool have_user_agent = false;
+    bool have_sra_release = false;
+    bool have_vdb_release = false;
+
     String accept_string;
+    String sra_release_string;
+    String vdb_release_string;
+    String user_agent_string;
+
     const KHttpHeader *node;
 
     if ( self == NULL ) {
@@ -1180,8 +1248,10 @@ rc_t CC KClientHttpRequestFormatMsgInt( const KClientHttpRequest *self,
          return RC ( rcNS, rcNoTarg, rcReading, rcParam, rcNull );
     }
 
-    CONST_STRING ( &user_agent_string, "User-Agent" );
-    CONST_STRING ( &accept_string, "Accept" );
+    CONST_STRING ( &accept_string     , "Accept" );        /*  6 */
+    CONST_STRING ( &sra_release_string, "X-SRA-Release" ); /* 13 */
+    CONST_STRING ( &vdb_release_string, "X-VDB-Release" ); /* 13 */
+    CONST_STRING ( &user_agent_string , "User-Agent" );    /* 10 */
 
     assert(method);
     if (method[0] != 'P') {
@@ -1213,6 +1283,16 @@ rc_t CC KClientHttpRequestFormatMsgInt( const KClientHttpRequest *self,
             if (StringCaseCompare(&node->name, &accept_string) == 0)
                 have_accept = true;
         }
+        /* look for "X-SRA-Release" */
+        else if (!have_sra_release && node->name.len == 13) {
+            if (StringCaseCompare(&node->name, &sra_release_string) == 0)
+                have_sra_release = true;
+        }
+        /* look for "X-VDB-Release" */
+        else if (!have_vdb_release && node->name.len == 13) {
+            if (StringCaseCompare(&node->name, &vdb_release_string) == 0)
+                have_vdb_release = true;
+        }
 
         /* add header line */
         rc = KDataBufferPrintf ( buffer, "%S: %S\r\n"
@@ -1220,6 +1300,39 @@ rc_t CC KClientHttpRequestFormatMsgInt( const KClientHttpRequest *self,
                                 , & node -> value );
     }
 
+  /* add an Accept header if we did not find one already in the header tree */
+    if (!have_accept) {
+        r2 = KDataBufferPrintf(buffer, "Accept: */*\r\n");
+        if (rc == 0 && r2 != 0)
+            rc = r2;
+    }
+
+    /* add a X-SRA-Release header if we did not find one
+       already in the header tree */
+    if (!have_sra_release) {
+        SraReleaseVersion version;
+        r2 = SraReleaseVersionGet(&version);
+        if (r2 == 0) {
+            r2 = KDataBufferPrintf(buffer, "X-SRA-Release: %V\r\n",
+                version.version);
+        }
+        if (rc == 0 && r2 != 0)
+            rc = r2;
+    }
+
+    /* add a X-VDB-Release header if we did not find one
+       already in the header tree */
+    if (!have_vdb_release) {
+        SraReleaseVersion version;
+        r2 = SraReleaseVersionGet(&version);
+        if (r2 == 0) {
+            r2 = KDataBufferPrintf(buffer, "X-VDB-Release: %V\r\n",
+                version.version);
+        }
+        if (rc == 0 && r2 != 0)
+            rc = r2;
+    }
+
     /* add an User-Agent header from the kns-manager if we did not find one already in the header tree */
     if ( !have_user_agent )
     {
@@ -1235,12 +1348,6 @@ rc_t CC KClientHttpRequestFormatMsgInt( const KClientHttpRequest *self,
         }
     }
 
-    if (!have_accept) {
-        r2 = KDataBufferPrintf(buffer, "Accept: */*\r\n");
-        if (rc == 0 && r2 != 0)
-            rc = r2;
-    }
-
     /* add terminating empty header line */
     if ( rc == 0 )
     {
@@ -1315,7 +1422,7 @@ rc_t KClientHttpRequestHandleRedirection ( KClientHttpRequest *self, const char
         if ( rc == 0 )
         {
             /* parse the URI into local url_block */
-            rc = ParseUrl ( &b, uri . base, uri . elem_count - 1 );
+            rc = ParseUrl ( &b, (char *) uri . base, uri . elem_count - 1 );
             if ( rc == 0 )
             {
                 KClientHttp *http = self -> http;
@@ -1379,13 +1486,15 @@ rc_t KClientHttpRequestSendReceiveNoBodyInt ( KClientHttpRequest *self, KClientH
             break;
 
         /* send the message and create a response */
-        rc = KClientHttpSendReceiveMsg ( self -> http, _rslt, buffer.base,
-            buffer.elem_count - 1, NULL, self -> url_buffer . base );
+        rc = KClientHttpSendReceiveMsg ( self -> http, _rslt,
+            (char *) buffer.base,
+            buffer.elem_count - 1, NULL, (char *) self -> url_buffer . base );
         if ( rc != 0 )
         {
             KClientHttpClose ( self -> http );
-            rc = KClientHttpSendReceiveMsg ( self -> http, _rslt, buffer.base,
-                buffer.elem_count - 1, NULL, self -> url_buffer . base );
+            rc = KClientHttpSendReceiveMsg ( self -> http, _rslt,
+                (char *) buffer.base, buffer.elem_count - 1, NULL,
+                (char *) self -> url_buffer . base );
             if ( rc != 0 )
                 break;
         }
@@ -1464,7 +1573,8 @@ static
 rc_t KClientHttpRequestSendReceiveNoBody ( KClientHttpRequest *self, KClientHttpResult **_rslt, const char *method )
 {
     KHttpRetrier retrier;
-    rc_t rc = KHttpRetrierInit ( & retrier, self -> url_buffer . base, self -> http -> mgr );
+    rc_t rc = KHttpRetrierInit ( & retrier,
+        (char *) self -> url_buffer . base, self -> http -> mgr );
 
     if ( rc == 0 )
     {
@@ -1632,11 +1742,15 @@ rc_t CC KClientHttpRequestPOST_Int ( KClientHttpRequest *self, KClientHttpResult
         }
 
         /* send the message and create a response */
-        rc = KClientHttpSendReceiveMsg ( self -> http, _rslt, buffer.base, buffer.elem_count, body, self -> url_buffer . base );
+        rc = KClientHttpSendReceiveMsg ( self -> http, _rslt,
+            (char *) buffer.base, buffer.elem_count, body,
+            (char *) self -> url_buffer . base );
         if ( rc != 0 )
         {
             KClientHttpClose ( self -> http );
-            rc = KClientHttpSendReceiveMsg ( self -> http, _rslt, buffer.base, buffer.elem_count, NULL, self -> url_buffer . base );
+            rc = KClientHttpSendReceiveMsg ( self -> http, _rslt,
+                (char *) buffer.base, buffer.elem_count, NULL,
+                (char *) self -> url_buffer . base );
         }
         KDataBufferWhack( & buffer );
         if ( rc != 0 )
@@ -1803,7 +1917,8 @@ LIB_EXPORT rc_t CC KClientHttpRequestPOST ( KClientHttpRequest *self, KClientHtt
         return RC ( rcNS, rcNoTarg, rcUpdating, rcParam, rcNull );
     }
 
-    rc = KHttpRetrierInit ( & retrier, self -> url_buffer . base, self -> http -> mgr );
+    rc = KHttpRetrierInit ( & retrier, (char *) self -> url_buffer . base,
+        self -> http -> mgr );
 
     if ( rc == 0 )
     {
@@ -1818,7 +1933,7 @@ LIB_EXPORT rc_t CC KClientHttpRequestPOST ( KClientHttpRequest *self, KClientHtt
             assert ( * _rslt );
 
             if ( ( * _rslt ) -> status  == 403 &&
-                 GovSiteByHttp ( self -> url_buffer . base ) )
+                 GovSiteByHttp ((char *) self -> url_buffer . base ) )
             {
                 break;
             }
