@@ -47,16 +47,17 @@
 
 #include <atomic.h> /* atomic_ptr_t */
 #include <ctype.h>
+#include <strtol.h> /* strtou64 */
 #include <sysalloc.h>
 
 #include <assert.h>
 
 #if LINUX
 #include <gnu/libc-version.h>
-#endif
-
 #include <arpa/inet.h>
 #include <netinet/in.h>
+#endif
+
 #include <stdio.h> /* fprintf */
 
 #include "../klib/release-vers.h"
@@ -88,6 +89,8 @@ static KDataBuffer kns_manager_pagehitid;
 static KDataBuffer kns_manager_ua_suffix;
 static KDataBuffer kns_manager_guid;
 
+quitting_t quitting;
+
 #if USE_SINGLETON
 static atomic_ptr_t kns_singleton;
 #endif
@@ -174,6 +177,8 @@ LIB_EXPORT rc_t CC KNSManagerRelease ( const KNSManager *self )
             return RC ( rcNS, rcMgr, rcAttaching, rcRefcount, rcInvalid );
         }
     }
+    else
+         return KDataBufferWhack(&kns_manager_user_agent);
     return 0;
 }
 
@@ -214,40 +219,57 @@ static rc_t CC KNSManagerMakeSingleton (
                 rc = KLockMake ( &kns_manager_lock );
                 if ( rc ) { return rc; }
             }
-            rc = KDataBufferMakeBytes ( &kns_manager_user_agent, 0 );
-            if ( rc ) { return rc; }
-            rc = KDataBufferPrintf ( &kns_manager_user_agent, "%s", "" );
-            if ( rc ) { return rc; }
 
-            rc = KDataBufferMakeBytes ( &kns_manager_user_agent_append, 0 );
-            if ( rc ) { return rc; }
-            rc = KDataBufferPrintf ( &kns_manager_user_agent_append, "%s", "" );
-            if ( rc ) { return rc; }
+            if (kns_manager_user_agent.base == NULL) {
+                rc = KDataBufferMakeBytes ( &kns_manager_user_agent, 0 );
+                if ( rc ) { return rc; }
+                rc = KDataBufferPrintf ( &kns_manager_user_agent, "%s", "" );
+                if ( rc ) { return rc; }
+            }
 
-            rc = KDataBufferMakeBytes ( &kns_manager_clientip, 0 );
-            if ( rc ) { return rc; }
-            rc = KDataBufferPrintf ( &kns_manager_clientip, "%s", "" );
-            if ( rc ) { return rc; }
+            if (kns_manager_user_agent_append.base == NULL) {
+                rc = KDataBufferMakeBytes ( &kns_manager_user_agent_append, 0 );
+                if ( rc ) { return rc; }
 
-            rc = KDataBufferMakeBytes ( &kns_manager_sessionid, 0 );
-            if ( rc ) { return rc; }
-            rc = KDataBufferPrintf ( &kns_manager_sessionid, "%s", "" );
-            if ( rc ) { return rc; }
+                rc = KDataBufferPrintf ( &kns_manager_user_agent_append, "%s",
+                                                                           "" );
+                if ( rc ) { return rc; }
+            }
 
-            rc = KDataBufferMakeBytes ( &kns_manager_pagehitid, 0 );
-            if ( rc ) { return rc; }
-            rc = KDataBufferPrintf ( &kns_manager_pagehitid, "%s", "" );
-            if ( rc ) { return rc; }
+            if (kns_manager_clientip.base == NULL) {
+                rc = KDataBufferMakeBytes ( &kns_manager_clientip, 0 );
+                if ( rc ) { return rc; }
+                rc = KDataBufferPrintf ( &kns_manager_clientip, "%s", "" );
+                if ( rc ) { return rc; }
+            }
 
-            rc = KDataBufferMakeBytes ( &kns_manager_ua_suffix, 0 );
-            if ( rc ) { return rc; }
-            rc = KDataBufferPrintf ( &kns_manager_ua_suffix, "%s", "" );
-            if ( rc ) { return rc; }
+            if (kns_manager_sessionid.base == NULL) {
+                rc = KDataBufferMakeBytes ( &kns_manager_sessionid, 0 );
+                if ( rc ) { return rc; }
+                rc = KDataBufferPrintf ( &kns_manager_sessionid, "%s", "" );
+                if ( rc ) { return rc; }
+            }
 
-            rc = KDataBufferMakeBytes ( &kns_manager_guid, 0 );
-            if ( rc ) { return rc; }
-            rc = KDataBufferPrintf ( &kns_manager_guid, "%s", "" );
-            if ( rc ) { return rc; }
+            if (kns_manager_pagehitid.base == NULL) {
+                rc = KDataBufferMakeBytes ( &kns_manager_pagehitid, 0 );
+                if ( rc ) { return rc; }
+                rc = KDataBufferPrintf ( &kns_manager_pagehitid, "%s", "" );
+                if ( rc ) { return rc; }
+            }
+
+            if (kns_manager_ua_suffix.base == NULL) {
+                rc = KDataBufferMakeBytes ( &kns_manager_ua_suffix, 0 );
+                if ( rc ) { return rc; }
+                rc = KDataBufferPrintf ( &kns_manager_ua_suffix, "%s", "" );
+                if ( rc ) { return rc; }
+            }
+
+            if (kns_manager_guid.base == NULL) {
+                rc = KDataBufferMakeBytes ( &kns_manager_guid, 0 );
+                if ( rc ) { return rc; }
+                rc = KDataBufferPrintf ( &kns_manager_guid, "%s", "" );
+                if ( rc ) { return rc; }
+            }
 
             rc = KNSManagerMakeConfigImpl ( &our_mgr, kfg );
 
@@ -592,7 +614,7 @@ static void KNSManagerSetNCBI_VDB_NET ( KNSManager *self, const KConfig *kfg )
 /* VDB-DESIREMENTS:
 1. to call *[s]/kfg/properties* to read configuration
 2. to create a header file to keep constants (node names) */
-static int32_t KNSManagerPrepareConnTimeout ( KConfig *kfg )
+static int32_t KNSManagerLoadConnTimeout ( KConfig *kfg )
 {
     int64_t result = 0;
     rc_t rc = KConfigReadI64 ( kfg, "/libs/kns/connect/timeout", &result );
@@ -600,7 +622,7 @@ static int32_t KNSManagerPrepareConnTimeout ( KConfig *kfg )
 
     return result;
 }
-static int32_t KNSManagerPrepareConnReadTimeout ( KConfig *kfg )
+static int32_t KNSManagerLoadConnReadTimeout ( KConfig *kfg )
 {
     int64_t result = 0;
     rc_t rc = KConfigReadI64 ( kfg, "/libs/kns/connect/timeout/read", &result );
@@ -608,7 +630,7 @@ static int32_t KNSManagerPrepareConnReadTimeout ( KConfig *kfg )
 
     return result;
 }
-static int32_t KNSManagerPrepareConnWriteTimeout ( KConfig *kfg )
+static int32_t KNSManagerLoadConnWriteTimeout ( KConfig *kfg )
 {
     int64_t result = 0;
     rc_t rc
@@ -618,7 +640,7 @@ static int32_t KNSManagerPrepareConnWriteTimeout ( KConfig *kfg )
     return result;
 }
 
-static int32_t KNSManagerPrepareHttpReadTimeout ( KConfig *kfg )
+static int32_t KNSManagerLoadHttpReadTimeout ( KConfig *kfg )
 {
     int64_t result = 0;
     rc_t rc = KConfigReadI64 ( kfg, "/http/timeout/read", &result );
@@ -626,7 +648,7 @@ static int32_t KNSManagerPrepareHttpReadTimeout ( KConfig *kfg )
 
     return result;
 }
-static int32_t KNSManagerPrepareHttpWriteTimeout ( KConfig *kfg )
+static int32_t KNSManagerLoadHttpWriteTimeout ( KConfig *kfg )
 {
     int64_t result = 0;
     rc_t rc = KConfigReadI64 ( kfg, "/http/timeout/write", &result );
@@ -635,8 +657,88 @@ static int32_t KNSManagerPrepareHttpWriteTimeout ( KConfig *kfg )
     return result;
 }
 
+static uint32_t KNSManagerLoadTotalWaitForReliableURLs(const KConfig *kfg)
+{
+    rc_t rc = 0;
+
+    int64_t result = 0;
+
+    const char * str = getenv("NCBI_VDB_RELIABLE_WAIT");
+    if (str != NULL) {
+        char *end = NULL;
+        result = strtou64(str, &end, 0);
+        if (end[0] == 0)
+            return result;
+    }
+
+    rc = KConfigReadI64(kfg, "/http/reliable/wait", &result);
+    if (rc != 0 || result < 0)
+        result = 10 * 60 * 1000; /* 10 min */
+
+    return result;
+}
+
+static bool KNSManagerLoadRetryFirstRead(const KConfig *kfg) {
+    rc_t rc = 0;
+
+    bool result = 0;
+
+    const char * str = getenv("NCBI_VDB_RELIABLE_RETRY_FIRST_READ");
+    if (str != NULL && str[0] != '\0') {
+        switch (str[0]) {
+        case 'f':
+            return false;
+        case 't':
+            return true;
+        default:
+            break;
+        }
+    }
+
+    rc = KConfigReadBool(kfg, "/http/reliable/retryFirstRead", &result);
+    if (rc != 0)
+        result = false;
+
+    return result;
+}
+
+static bool KNSManagerLoadRetryFile(const KConfig *kfg) {
+    rc_t rc = 0;
+
+    bool result = 0;
+
+    const char * str = getenv("NCBI_VDB_RELIABLE_RETRY_FILE");
+    if (str != NULL && str[0] != '\0') {
+        switch (str[0]) {
+        case 'f':
+            return false;
+        case 't':
+            return true;
+        default:
+            break;
+        }
+    }
+
+    rc = KConfigReadBool(kfg, "/http/reliable/retryFile", &result);
+    if (rc != 0)
+        result = true;
+
+    return result;
+}
+
+static uint8_t KNSManagerLoadMaxNumberOfRetriesOnFailureForReliableURLs
+(const KConfig *kfg)
+{
+    int64_t result = 0;
+    rc_t rc = KConfigReadI64(kfg, "/http/reliable/retries", &result);
+    if (rc != 0 || result < 0)
+        result = 10;
+
+    return result;
+}
+
 #if 0
-static bool KNSManagerPrepareLogTlsErrors(KConfig* kfg) {
+static bool KNSManagerLoadLogTlsErrors(KConfig* kfg) {
     const char * e = getenv("NCBI_VDB_TLS_LOG_ERR");
     if (e != NULL)
         if (e[0] == '\0')
@@ -660,7 +762,7 @@ static bool KNSManagerPrepareLogTlsErrors(KConfig* kfg) {
     }
 }
 
-static int KNSManagerPrepareEmulateTldReadErrors(KConfig* kfg) {
+static int KNSManagerLoadEmulateTldReadErrors(KConfig* kfg) {
     const char * e = getenv("NCBI_VDB_ERR_MBEDTLS_READ");
     if (e != NULL)
         return atoi(e);
@@ -675,7 +777,7 @@ static int KNSManagerPrepareEmulateTldReadErrors(KConfig* kfg) {
 }
 #endif
 
-static bool KNSManagerPrepareResolveToCache ( KConfig *kfg )
+static bool KNSManagerLoadResolveToCache ( KConfig *kfg )
 {
     /* VResolverCache resolve to user's cache vs. cwd/AD */
     bool reslt = true;
@@ -688,7 +790,7 @@ static bool KNSManagerPrepareResolveToCache ( KConfig *kfg )
     return true;
 }
 
-static bool KNSManagerPrepareAcceptAwsCharges ( KConfig *kfg )
+static bool KNSManagerLoadAcceptAwsCharges ( KConfig *kfg )
 {
     bool reslt = false;
 
@@ -699,7 +801,7 @@ static bool KNSManagerPrepareAcceptAwsCharges ( KConfig *kfg )
     return false;
 }
 
-static bool KNSManagerPrepareAcceptGcpCharges ( KConfig *kfg )
+static bool KNSManagerLoadAcceptGcpCharges ( KConfig *kfg )
 {
     bool reslt = false;
 
@@ -722,22 +824,28 @@ static rc_t CC KNSManagerMakeConfigImpl ( KNSManager **mgrp, KConfig *kfg )
             rc = RC ( rcNS, rcMgr, rcAllocating, rcMemory, rcExhausted );
         } else {
             KRefcountInit ( &mgr->refcount, 1, "KNSManager", "init", "kns" );
-            mgr->conn_timeout = KNSManagerPrepareConnTimeout ( kfg );
-            mgr->conn_read_timeout = KNSManagerPrepareConnReadTimeout ( kfg );
-            mgr->conn_write_timeout = KNSManagerPrepareConnWriteTimeout ( kfg );
-            mgr->http_read_timeout = KNSManagerPrepareHttpReadTimeout ( kfg );
-            mgr->http_write_timeout = KNSManagerPrepareHttpWriteTimeout ( kfg );
-            mgr->maxTotalWaitForReliableURLs_ms = 10 * 60 * 1000; /* 10 min */
-            mgr->maxNumberOfRetriesOnFailureForReliableURLs = 10;
 
-            /*          mgr->logTlsErrors = KNSManagerPrepareLogTlsErrors(kfg);
+            mgr->conn_timeout = KNSManagerLoadConnTimeout ( kfg );
+            mgr->conn_read_timeout = KNSManagerLoadConnReadTimeout ( kfg );
+            mgr->conn_write_timeout = KNSManagerLoadConnWriteTimeout ( kfg );
+            mgr->http_read_timeout = KNSManagerLoadHttpReadTimeout ( kfg );
+            mgr->http_write_timeout = KNSManagerLoadHttpWriteTimeout ( kfg );
+            mgr->maxTotalWaitForReliableURLs_ms
+                = KNSManagerLoadTotalWaitForReliableURLs ( kfg );
+            mgr->maxNumberOfRetriesOnFailureForReliableURLs
+                = KNSManagerLoadMaxNumberOfRetriesOnFailureForReliableURLs (
+                    kfg );
+            mgr->retryFirstRead = KNSManagerLoadRetryFirstRead(kfg);
+            mgr->retryFile = KNSManagerLoadRetryFile ( kfg );
+            mgr->max_http_read_timeout = 60 * 1000; /* 1 minute */
+            /*          mgr->logTlsErrors = KNSManagerLoadLogTlsErrors(kfg);
                         mgr->emulateTlsReadErrors
-                            = KNSManagerPrepareEmulateTldReadErrors(kfg); */
+                            = KNSManagerLoadEmulateTldReadErrors(kfg); */
 
-            mgr->resolveToCache = KNSManagerPrepareResolveToCache ( kfg );
+            mgr->resolveToCache = KNSManagerLoadResolveToCache ( kfg );
 
-            mgr->accept_aws_charges = KNSManagerPrepareAcceptAwsCharges ( kfg );
-            mgr->accept_gcp_charges = KNSManagerPrepareAcceptGcpCharges ( kfg );
+            mgr->accept_aws_charges = KNSManagerLoadAcceptAwsCharges ( kfg );
+            mgr->accept_gcp_charges = KNSManagerLoadAcceptGcpCharges ( kfg );
 
             if ( strlen ( kns_manager_guid.base ) == 0 ) {
                 rc = KDataBufferResize ( &kns_manager_guid, 37 );
@@ -1127,3 +1235,127 @@ LIB_EXPORT rc_t CC KNSManagerSetPageHitID (
 
     return rc;
 }
+
+
+LIB_EXPORT
+rc_t CC KNSManagerSetQuitting(KNSManager *self, quitting_t aQuitting)
+{
+    quitting = aQuitting;
+    return 0;
+}
+
+LIB_EXPORT
+quitting_t CC KNSManagerGetQuitting(const KNSManager *self)
+{
+    return quitting;
+}
+
+/******************************************************************************/
+/**************** API to manage HTTP File read retry behavior *****************/
+/******************************************************************************/
+
+/* SetRetryFailedReads
+ *  manages retry layer on HttpFileRead
+ *
+ *  "retry" [ IN ] - true : turn on retry layer,
+ *                   false: don't create retry layer.
+ */
+KNS_EXTERN rc_t CC KNSManagerSetRetryFailedReads(KNSManager *self,
+    bool retry)
+{
+    if (self == NULL)
+        return RC(rcNS, rcMgr, rcUpdating, rcSelf, rcNull);
+    else {
+        self->retryFile = retry;
+        return 0;
+    }
+}
+
+/* GetRetryFailedReads
+ *  returns whether or not retry layer on HttpFileRead is turned on
+ */
+LIB_EXPORT rc_t CC KNSManagerGetRetryFailedReads(const KNSManager *self,
+    bool *retry)
+{
+    if (self == NULL)
+        return RC(rcNS, rcMgr, rcAccessing, rcSelf, rcNull);
+    else if (retry == NULL)
+        return RC(rcNS, rcMgr, rcAccessing, rcParam, rcNull);
+    else {
+        *retry = self->retryFile;
+        return 0;
+    }
+}
+
+/* SetMaxReadRetryTime
+ *  sets maximum time in HttpFileRead retry loop
+ *
+ *  "millis" - when negative, infinite timeout
+ */
+LIB_EXPORT rc_t CC KNSManagerSetMaxReadRetryTime(KNSManager *self,
+    int32_t millis)
+{
+    if (self == NULL)
+        return RC(rcNS, rcMgr, rcUpdating, rcSelf, rcNull);
+    else {
+        if (millis < 0)
+            self->maxTotalWaitForReliableURLs_ms = ~0;
+        else
+            self->maxTotalWaitForReliableURLs_ms = millis;
+
+        return 0;
+    }
+}
+
+/* GetMaxReadRetryTime
+ *  returns maximum time in HttpFileRead retry loop
+ */
+LIB_EXPORT rc_t CC KNSManagerGetMaxReadRetryTime(const KNSManager *self,
+    int32_t *millis)
+{
+    if (self == NULL)
+        return RC(rcNS, rcMgr, rcAccessing, rcSelf, rcNull);
+    else if (millis == NULL)
+        return RC(rcNS, rcMgr, rcAccessing, rcParam, rcNull);
+    else {
+        if (self->maxTotalWaitForReliableURLs_ms == ~0)
+            *millis = -1;
+        else
+            *millis = self->maxTotalWaitForReliableURLs_ms;
+
+        return 0;
+    }
+}
+
+
+/* SetRetryFirstReads
+ *  manages retry on the first HttpFileRead
+ */
+LIB_EXPORT rc_t CC KNSManagerSetRetryFirstReads(KNSManager *self,
+    bool retry)
+{
+    if (self == NULL)
+        return RC(rcNS, rcMgr, rcUpdating, rcSelf, rcNull);
+    else {
+        self->retryFirstRead = retry;
+        return 0;
+    }
+}
+
+/* GetRetryFirstReads
+ *  returns whether or not retry on the first HttpFileRead is turned on
+ */
+LIB_EXPORT rc_t CC KNSManagerGetRetryFirstReads(const KNSManager *self,
+    bool *retry)
+{
+    if (self == NULL)
+        return RC(rcNS, rcMgr, rcAccessing, rcSelf, rcNull);
+    else if (retry == NULL)
+        return RC(rcNS, rcMgr, rcAccessing, rcParam, rcNull);
+    else {
+        *retry = self->retryFirstRead;
+        return 0;
+    }
+}
+
+/******************************************************************************/
