@@ -41,6 +41,7 @@ struct KTLSStream;
 #include <klib/namelist.h>
 #include <kproc/timeout.h>
 #include <kfg/config.h>
+#include <kfs/directory.h>
 
 #include <os-native.h>
 
@@ -56,7 +57,7 @@ struct KTLSStream;
 #if ! defined ( MBEDTLS_CONFIG_FILE )
 #include <mbedtls/config.h>
 #else
-#include MBEDTLS_CONFIG_FILE 
+#include MBEDTLS_CONFIG_FILE
 #endif
 
 #include <mbedtls/net.h>
@@ -68,6 +69,10 @@ struct KTLSStream;
 #include <mbedtls/error.h>
 #include <mbedtls/certs.h>
 
+#if WINDOWS
+#define IGNORE_ALL_CERTS_ALLOWED 1
+#endif
+
 static const char ca_crt_ncbi1 [] =
     "-----BEGIN CERTIFICATE-----\r\n"
     "MIIDrzCCApegAwIBAgIQCDvgVpBCRrGhdWrJWZHHSjANBgkqhkiG9w0BAQUFADBh\r\n"
@@ -178,7 +183,7 @@ rc_t tlsg_seed_rng ( KTLSGlobals *self )
 
     STATUS ( STAT_QA, "Seeding the random number generator\n" );
 
-    ret = vdb_mbedtls_ctr_drbg_seed ( &self -> ctr_drbg, vdb_mbedtls_entropy_func, 
+    ret = vdb_mbedtls_ctr_drbg_seed ( &self -> ctr_drbg, vdb_mbedtls_entropy_func,
                                   &self -> entropy, ( const unsigned char * ) pers, pers_size );
 
     if ( ret != 0 )
@@ -192,13 +197,14 @@ rc_t tlsg_seed_rng ( KTLSGlobals *self )
                       ) );
         return rc;
     }
-   
+
     return 0;
 }
 
 static
 rc_t tlsg_init_ca ( KTLSGlobals *self, const KConfig * kfg )
 {
+#if IGNORE_ALL_CERTS_ALLOWED
     const KConfigNode * allow_all_certs;
     rc_t rc = KConfigOpenNodeRead ( kfg, & allow_all_certs, "/tls/allow-all-certs" );
     if ( rc == 0 )
@@ -207,16 +213,17 @@ rc_t tlsg_init_ca ( KTLSGlobals *self, const KConfig * kfg )
         KConfigNodeRelease ( allow_all_certs );
         return rc;
     }
-
+#endif
     /* if the node does not exist, it means false */
     self -> allow_all_certs = false;
     return 0;
 }
 
-static 
+static
 rc_t tlsg_init_certs ( KTLSGlobals *self, const KConfig * kfg )
 {
-    int ret;
+    int ret = 0;
+    bool cert_file_loaded = false;
 
     rc_t rc = 0;
     uint32_t nidx, num_certs = 0;
@@ -292,7 +299,7 @@ rc_t tlsg_init_certs ( KTLSGlobals *self, const KConfig * kfg )
                                           ) );
                             break;
                         }
-                        
+
                         STATUS ( STAT_GEEK, "Retrieving node '%s' from CA root certificates\n", cert_name );
                         rc = KConfigNodeOpenNodeRead ( ca_crt, & root_cert, "%s", cert_name );
                         if ( rc != 0 )
@@ -306,11 +313,11 @@ rc_t tlsg_init_certs ( KTLSGlobals *self, const KConfig * kfg )
                                           ) );
                             break;
                         }
-                        
+
                         STATUS ( STAT_GEEK, "Retrieving text for node '%s' from CA root certificates\n", cert_name );
                         rc = KConfigNodeReadString ( root_cert, & cert_string );
                         KConfigNodeRelease ( root_cert );
-                        
+
                         if ( rc != 0 )
                         {
                             rc = ResetRCContext ( rc, rcKrypto, rcToken, rcInitializing );
@@ -329,11 +336,11 @@ rc_t tlsg_init_certs ( KTLSGlobals *self, const KConfig * kfg )
                         STATUS ( STAT_GEEK, "Parsing text for node '%s' from CA root certificates\n", cert_name );
                         ret = vdb_mbedtls_x509_crt_parse ( &self -> cacert,
                             ( const unsigned char * ) cert_string -> addr, cert_string -> size + 1 );
-                    
+
                         StringWhack ( cert_string );
-                    
+
                         if ( ret < 0 )
-                        {        
+                        {
                             rc = RC ( rcKrypto, rcToken, rcInitializing, rcEncryption, rcFailed );
                             PLOGERR ( klogSys, ( klogSys, rc
                                                  , "mbedtls_x509_crt_parse returned $(ret) ( $(expl) )"
@@ -343,11 +350,11 @@ rc_t tlsg_init_certs ( KTLSGlobals *self, const KConfig * kfg )
                                           ) );
                             break;
                         }
-                        
+
                         ++ num_certs;
                     }
                 }
-            
+
                 KNamelistRelease ( cert_names );
             }
 
@@ -375,7 +382,9 @@ rc_t tlsg_init_certs ( KTLSGlobals *self, const KConfig * kfg )
             {
                 STATUS ( STAT_GEEK, "Parsing text from CA root certificate file '%S'\n", ca_crt_path );
                 ret = vdb_mbedtls_x509_crt_parse_file ( &self -> cacert, ca_crt_path -> addr );
-                if ( ret < 0 )
+                if ( ret >= 0 )
+                    cert_file_loaded = true;
+                else
                 {
                     PLOGMSG ( klogWarn, ( klogWarn
                                           , "mbedtls_x509_crt_parse_file ( '$(path)' ) returned $(ret) ( $(expl) )"
@@ -384,7 +393,7 @@ rc_t tlsg_init_certs ( KTLSGlobals *self, const KConfig * kfg )
                                           , mbedtls_strerror2 ( ret )
                                           , ca_crt_path
                                   ) );
-                }                
+                }
                 StringWhack ( ca_crt_path );
             }
             KConfigNodeRelease ( ca_crt );
@@ -392,14 +401,86 @@ rc_t tlsg_init_certs ( KTLSGlobals *self, const KConfig * kfg )
     }
 #endif
 
+    if ( ! cert_file_loaded )
+    {
+#if WINDOWS
+        //-------------------------------------------------------------------
+        // Pull all the certificates from the ROOT store.
+
+        HCERTSTORE hSystemStore = CertOpenSystemStoreA(0, "ROOT");
+        if ( hSystemStore != NULL )
+        {
+            PCCERT_CONTEXT pCertContext = NULL;
+            while (true)
+            {
+                pCertContext = CertEnumCertificatesInStore(hSystemStore, pCertContext);
+                if (pCertContext == NULL)
+                {
+                    break;
+                }
+
+                // ignore errors
+                if ( vdb_mbedtls_x509_crt_parse(&self->cacert, (const unsigned char*)pCertContext->pbCertEncoded, pCertContext->cbCertEncoded ) == 0 )
+                {
+                    cert_file_loaded = true;
+                }
+            }
+            CertCloseStore(hSystemStore, 0);
+        }
+#else
+        const char * root_ca_paths [] =
+        {
+            "/etc/ssl/certs/ca-certificates.crt",                /* Debian/Ubuntu/Gentoo etc */
+            "/etc/pki/tls/certs/ca-bundle.crt",                  /* Fedora/RHEL */
+            "/etc/ssl/ca-bundle.pem",                            /* OpenSUSE */
+            "/etc/pki/tls/cacert.pem",                           /* OpenELEC */
+            "/etc/pki/ca-trust/extracted/pem/tls-ca-bundle.pem", /* CentOS/RHEL 7 */
+            "/etc/ssl/certs",                                    /* SLES10/SLES11, https://golang.org/issue/12139 */
+            "/system/etc/security/cacerts",                      /* Android */
+            "/usr/local/share/certs",                            /* FreeBSD */
+            "/etc/pki/tls/certs",                                /* Fedora/RHEL */
+            "/etc/openssl/certs",                                /* NetBSD */
+            "/etc/ssl/cert.pem"                                  /* MacOSX */
+        };
+
+        KDirectory *n_dir;
+        rc_t rc2 = KDirectoryNativeDir ( & n_dir );
+        if ( rc2 == 0 )
+        {
+            size_t i;
+            for ( ret = -1, i = 0; i < sizeof root_ca_paths / sizeof root_ca_paths [ 0 ]; ++ i )
+            {
+                const char * path = root_ca_paths [ i ];
+
+                /* no fail, exit immediately if succesful or cycle through all files */
+                switch ( KDirectoryPathType ( n_dir, path ) & ~ kptAlias )
+                {
+                case kptFile:
+                    STATUS ( STAT_GEEK, "Parsing text from CA root certificate file '%s'\n", path);
+                    ret = vdb_mbedtls_x509_crt_parse_file ( &self -> cacert, path );
+                    break;
+                case kptDir:
+                    STATUS ( STAT_GEEK, "Parsing text from CA root certificate directory '%s'\n", path );
+                    ret = vdb_mbedtls_x509_crt_parse_path ( &self -> cacert, path );
+                    break;
+                }
+
+                if ( ret >= 0 )
+                    cert_file_loaded = true;
+            }
+            KDirectoryRelease ( n_dir );
+        }
+#endif
+    }
+
     if ( num_certs == 0 )
     {
         STATUS ( STAT_QA, "Parsing text for default CA root certificates\n" );
         ret = vdb_mbedtls_x509_crt_parse ( &self -> cacert,
             ( const unsigned char * ) ca_crt_ncbi1, sizeof ca_crt_ncbi1 );
-                    
+
         if ( ret < 0 )
-        {        
+        {
             rc = RC ( rcKrypto, rcToken, rcInitializing, rcEncryption, rcFailed );
             PLOGERR ( klogSys, ( klogSys, rc
                                  , "mbedtls_x509_crt_parse returned $(ret) ( $(expl) )"
@@ -411,7 +492,7 @@ rc_t tlsg_init_certs ( KTLSGlobals *self, const KConfig * kfg )
         else
         {
             num_certs = 1;
-            
+
             ret = vdb_mbedtls_x509_crt_parse ( &self -> cacert,
                 ( const unsigned char * ) ca_crt_ncbi2, sizeof ca_crt_ncbi2 );
 
@@ -465,23 +546,32 @@ rc_t tlsg_setup ( KTLSGlobals * self )
     vdb_mbedtls_ssl_conf_ca_chain( &self -> config, &self -> cacert, NULL );
     vdb_mbedtls_ssl_conf_rng( &self -> config, vdb_mbedtls_ctr_drbg_random, &self -> ctr_drbg );
 
-        /*  We need that to be sure that we are free to call 
-         *  vdb_mbedtls_ssl_conf_authmode () next time when 
+        /*  We need that to be sure that we are free to call
+         *  vdb_mbedtls_ssl_conf_authmode () next time when
          *  KNSManagerSetAllowAllCerts () will be called
-         *  
+         *
          *  Because smart special design we do not need to add
-         *  special code to deinitialize that variable. 
-         */ 
+         *  special code to deinitialize that variable.
+         */
     self -> safe_to_modify_ssl_config = true;
 
     return 0;
 }
 
+/* threshold
+   theshold level of messages to filter on.
+   Messages at a higher level will be discarded.
+  Debug levels
+	0 No debug
+	1 Error
+	2 State change
+	3 Informational
+	4 Verbose */
 static int set_threshold ( const KConfig * kfg ) {
     bool set = false;
 
     int64_t threshold = 0;
-    
+
     const char * env = NULL;
 
     rc_t rc = KConfigReadI64 ( kfg, "/tls/NCBI_VDB_TLS", & threshold );
@@ -545,7 +635,7 @@ rc_t KTLSGlobalsInit ( KTLSGlobals * tlsg, const KConfig * kfg )
 void KTLSGlobalsWhack ( KTLSGlobals * self )
 {
     assert ( self != NULL );
-    
+
     vdb_mbedtls_ssl_config_free ( &self -> config );
     vdb_mbedtls_entropy_free ( &self -> entropy );
     vdb_mbedtls_ctr_drbg_free ( &self -> ctr_drbg );
@@ -560,16 +650,17 @@ void KTLSGlobalsWhack ( KTLSGlobals * self )
 LIB_EXPORT rc_t CC KNSManagerSetAllowAllCerts ( KNSManager *self, bool allow_all_certs )
 {
     rc_t rc = 0;
-    
+
     if ( self == NULL )
         rc = RC ( rcNS, rcMgr, rcAccessing, rcSelf, rcNull );
     else
     {
+#if IGNORE_ALL_CERTS_ALLOWED
         self -> tlsg . allow_all_certs = allow_all_certs;
             /*
              *  We are acting from supposition that at some particular
-             *  moments there should be called initlialisation of 
-             *  TLS configurations, which will be reflected at next 
+             *  moments there should be called initlialisation of
+             *  TLS configurations, which will be reflected at next
              *  handshake
              */
         if ( self -> tlsg . safe_to_modify_ssl_config ) {
@@ -581,8 +672,8 @@ LIB_EXPORT rc_t CC KNSManagerSetAllowAllCerts ( KNSManager *self, bool allow_all
                                 )
                             );
         }
+#endif
     }
-
     return rc;
 }
 
@@ -601,7 +692,7 @@ LIB_EXPORT rc_t CC KNSManagerGetAllowAllCerts ( const KNSManager *self, bool * a
             * allow_all_certs = self -> tlsg . allow_all_certs;
             return 0;
         }
-        
+
         * allow_all_certs = false;
     }
 
@@ -655,6 +746,7 @@ rc_t CC KTLSStreamWhack ( KTLSStream *self )
     self -> mgr = NULL;
 
     /* done */
+    KStreamWhack ( & self -> dad, "KTLSStream" );
     free ( self );
     return 0;
 }
@@ -679,9 +771,35 @@ rc_t CC KTLSStreamRead ( const KTLSStream * cself,
 
     while ( 1 )
     {
+        static bool inited = false;
+        static int m = 0;
+        static int e = 0;
+
         /* read through TLS library */
         ret = vdb_mbedtls_ssl_read( &self -> ssl, buffer, bsize );
 
+        if (!inited) { /* simulate mbedtls read timeout */
+            const char * v = getenv("NCBI_VDB_ERR_MBEDTLS_READ");
+            if (v != NULL) {
+                m = atoi(v);
+                if (m < 0)
+                    m = 0;
+            }
+            e = m;
+            inited = true;
+        }
+        if (m > 0) {
+            if (!e) {
+                e = m;
+                if (ret >= 0) {
+                    ret = -76;
+                    self->rd_rc
+                        = RC(rcNS, rcStream, rcReading, rcTimeout, rcExhausted);
+                }
+            }
+            --e;
+        }
+
         /* no error */
         if ( ret >= 0 )
         {
@@ -693,7 +811,8 @@ rc_t CC KTLSStreamRead ( const KTLSStream * cself,
         if ( self -> rd_rc != 0 )
         {
             rc = self -> rd_rc;
-            PLOGERR ( klogSys, ( klogSys, rc
+            if (self->mgr->logTlsErrors)
+              PLOGERR ( klogSys, ( klogSys, rc
                                  , "mbedtls_ssl_read returned $(ret) ( $(expl) )"
                                  , "ret=%d,expl=%s"
                                  , ret
@@ -704,7 +823,7 @@ rc_t CC KTLSStreamRead ( const KTLSStream * cself,
             self -> rd_rc = 0;
             break;
         }
-    
+
         /* this is a TLS error */
         switch ( ret )
         {
@@ -713,10 +832,10 @@ rc_t CC KTLSStreamRead ( const KTLSStream * cself,
              * vdb_mbedtls_ssl_session_reset () before a new connection; current connection
              * must be closed
              */
-        case MBEDTLS_ERR_SSL_WANT_READ: 
+        case MBEDTLS_ERR_SSL_WANT_READ:
         case MBEDTLS_ERR_SSL_WANT_WRITE:
             continue; /* TBD - allow the app to check signals */
-        case MBEDTLS_ERR_SSL_CLIENT_RECONNECT: 
+        case MBEDTLS_ERR_SSL_CLIENT_RECONNECT:
             /* can only happen server-side:   When this function return MBEDTLS_ERR_SSL_CLIENT_RECONNECT
              * (which can only happen server-side), it means that a client
              * is initiating a new connection using the same source port.
@@ -771,12 +890,12 @@ rc_t CC KTLSStreamWrite ( KTLSStream * self,
     }
 
     STATUS ( STAT_PRG, "Writing %zu bytes to to server\n", size );
-    
+
     self -> wr_rc = 0;
 
     while ( 1 )
     {
-        /* write through TLS library 
+        /* write through TLS library
         *  This function will do partial writes in some cases. If the
         *  return value is non-negative but less than length, the
         *  function must be called again with updated arguments:
@@ -784,7 +903,7 @@ rc_t CC KTLSStreamWrite ( KTLSStream * self,
         *  it returns a value equal to the last 'len' argument.
         *
         *  We expect to be called through KStreamWriteAll that will
-        *  avoid the issue above. 
+        *  avoid the issue above.
         */
         ret = vdb_mbedtls_ssl_write ( &self -> ssl, buffer, size );
 
@@ -934,7 +1053,7 @@ int CC ktls_net_recv ( void *ctx, unsigned char *buf, size_t len )
     return ( int ) num_read;
 }
 
-static 
+static
 rc_t ktls_ssl_setup ( KTLSStream *self, const String *host )
 {
     int ret;
@@ -1010,17 +1129,46 @@ rc_t ktls_ssl_setup ( KTLSStream *self, const String *host )
     return 0;
 }
 
-static 
+static
 rc_t ktls_handshake ( KTLSStream *self )
 {
+    static bool inited = false;
+    static int m = 0;
+    static int e = 0;
+
     int ret;
 
     STATUS ( STAT_QA, "Performing SSL/TLS handshake...\n" );
 
+    assert(self && self->mgr);
+
     ret = vdb_mbedtls_ssl_handshake( &self -> ssl );
+
+    if (!inited) { /* simulate mbedtls handshake timeout */
+        const char * v = getenv("NCBI_VDB_ERR_MBEDTLS_HANDSHAKE");
+        if (v != NULL) {
+            m = atoi(v);
+            if (m < 0)
+                m = 0;
+        }
+        e = m;
+        inited = true;
+    }
+    if (m > 0) {
+        if (!e) {
+            e = m;
+            if (ret >= 0) {
+                ret = -76;
+                self->rd_rc
+                    = RC(rcKrypto, rcFile, rcOpening, rcConnection, rcFailed);
+            }
+        }
+        --e;
+    }
+
     while ( ret != 0 )
     {
-        if ( ret != MBEDTLS_ERR_SSL_WANT_READ && 
+        if ( ret != MBEDTLS_ERR_SSL_WANT_READ &&
              ret != MBEDTLS_ERR_SSL_WANT_WRITE )
         {
             rc_t rc;
@@ -1040,7 +1188,8 @@ rc_t ktls_handshake ( KTLSStream *self )
                    or the error is something other than a validation error */
                 rc = RC ( rcKrypto, rcSocket, rcOpening, rcConnection, rcFailed );
 
-                PLOGERR ( klogSys, ( klogSys, rc
+                if (self->mgr->logTlsErrors)
+                  PLOGERR ( klogSys, ( klogSys, rc
                                      , "mbedtls_ssl_handshake returned $(ret) ( $(expl) )"
                                      , "ret=%d,expl=%s"
                                      , ret
@@ -1054,7 +1203,7 @@ rc_t ktls_handshake ( KTLSStream *self )
                     {
                         char buf [ 4096 ];
                         vdb_mbedtls_x509_crt_verify_info ( buf, sizeof( buf ), " !! ", flags );
-                        
+
                         PLOGMSG ( klogSys, ( klogSys
                                              , "mbedtls_ssl_get_verify_result returned $(flags) ( $(info) )"
                                              , "flags=0x%X,info=%s"
@@ -1240,7 +1389,7 @@ LIB_EXPORT rc_t CC KTLSStreamVerifyCACert ( const KTLSStream * self )
     rc_t rc = 0;
 
    STATUS ( STAT_QA, "Verifying peer X.509 certificate..." );
-   
+
    if ( self == NULL )
        rc = RC ( rcKrypto, rcToken, rcValidating, rcSelf, rcNull );
    else if ( ! self -> mgr -> tlsg . allow_all_certs )
@@ -1251,7 +1400,7 @@ LIB_EXPORT rc_t CC KTLSStreamVerifyCACert ( const KTLSStream * self )
            char buf [ 4096 ];
            rc_t rc = RC ( rcKrypto, rcToken, rcValidating, rcEncryption, rcFailed );
 
-           vdb_mbedtls_x509_crt_verify_info ( buf, sizeof( buf ), "  ! ", flags );        
+           vdb_mbedtls_x509_crt_verify_info ( buf, sizeof( buf ), "  ! ", flags );
 
            PLOGERR ( klogSys, ( klogSys, rc
                                 , "mbedtls_ssl_get_verify_result returned $(flags) ( $(info) )"
@@ -1262,7 +1411,7 @@ LIB_EXPORT rc_t CC KTLSStreamVerifyCACert ( const KTLSStream * self )
            return rc;
        }
    }
-   
+
    return rc;
 }
 
