@@ -320,7 +320,7 @@ rc_t VDBManagerConfigFromKfg ( VDBManager *self, bool update )
             rc = VDBManagerGetKfgPath ( kfg, "vdb/schema/paths", full, sizeof full, & num_read );
             if ( rc == 0 )
             {
-                PLOGMSG ( klogDebug, ( klogDebug, "VDBManagerConfigFromKfg: vdb/schema/paths = '$(path)'", "path=%s", full ) );
+                DEBUG_PRINT ( "VDBManagerConfigFromKfg: vdb/schema/paths = '%s'", full );
             }
         }
         if ( rc != 0 )
