@@ -645,6 +645,21 @@ FIXTURE_TEST_CASE(ConfigNodeAccessInt, KfgFixture)
     int64_t i = 0;
     REQUIRE_RC(KConfigNodeReadI64(GetNode("int/i1"), &i));
     REQUIRE_EQ(i, (int64_t)100);
+    REQUIRE_RC_FAIL(KConfigNodeReadI64(GetNode("int"), &i));
+}
+
+FIXTURE_TEST_CASE(ConfigNodeAccessInts, KfgFixture)
+{
+    const char* contents =
+        "int/i1=\"100\"\n"
+        "int=\"200\"\n"
+        ;
+    CreateAndLoad(GetName(), contents);
+    int64_t i = 0;
+    REQUIRE_RC(KConfigNodeReadI64(GetNode("int/i1"), &i));
+    REQUIRE_EQ(i, (int64_t)100);
+    REQUIRE_RC(KConfigNodeReadI64(GetNode("int"), &i));
+    REQUIRE_EQ(i, (int64_t)200);
 }
 
 FIXTURE_TEST_CASE(ConfigNodeAccessUnsigned, KfgFixture)
@@ -656,6 +671,21 @@ FIXTURE_TEST_CASE(ConfigNodeAccessUnsigned, KfgFixture)
     uint64_t i = 0;
     REQUIRE_RC(KConfigNodeReadU64(GetNode("uint/i1"), &i));
     REQUIRE_EQ(i, UINT64_C(100000000000));
+    REQUIRE_RC_FAIL(KConfigNodeReadU64(GetNode("uint"), &i));
+}
+
+FIXTURE_TEST_CASE(ConfigNodeAccessUnsigneds, KfgFixture)
+{
+    const char* contents =
+        "uint/i1=\"100000000000\"\n"
+        "uint=\"200000000000\"\n"
+        ;
+    CreateAndLoad(GetName(), contents);
+    uint64_t i = 0;
+    REQUIRE_RC(KConfigNodeReadU64(GetNode("uint/i1"), &i));
+    REQUIRE_EQ(i, UINT64_C(100000000000));
+    REQUIRE_RC(KConfigNodeReadU64(GetNode("uint"), &i));
+    REQUIRE_EQ(i, UINT64_C(200000000000));
 }
 
 FIXTURE_TEST_CASE(ConfigNodeAccessF64, KfgFixture)
@@ -667,6 +697,21 @@ FIXTURE_TEST_CASE(ConfigNodeAccessF64, KfgFixture)
     double f = 0.0;
     REQUIRE_RC(KConfigNodeReadF64(GetNode("f64/i1"), &f));
     REQUIRE_CLOSE(f, 3.14, 0.001);
+    REQUIRE_RC_FAIL(KConfigNodeReadF64(GetNode("f64"), &f));
+}
+
+FIXTURE_TEST_CASE(ConfigNodeAccessF64s, KfgFixture)
+{
+    const char* contents =
+        "f64/i1=\"3.14\"\n"
+        "f64=\"6.14\"\n"
+        ;
+    CreateAndLoad(GetName(), contents);
+    double f = 0.0;
+    REQUIRE_RC(KConfigNodeReadF64(GetNode("f64/i1"), &f));
+    REQUIRE_CLOSE(f, 3.14, 0.001);
+    REQUIRE_RC(KConfigNodeReadF64(GetNode("f64"), &f));
+    REQUIRE_CLOSE(f, 6.14, 0.001);
 }
 
 FIXTURE_TEST_CASE(ConfigNodeAccessVPath, KfgFixture)
