@@ -64,8 +64,11 @@ using namespace ncbi::NK;
 #define RELEASE(type, obj) do { rc_t rc2 = type##Release(obj); \
     if (rc2 != 0 && rc == 0) { rc = rc2; } obj = NULL; } while (false)
 
+#define ALL
+
 //////////////////////////
 // Regular HTTP
+#ifdef ALL
 FIXTURE_TEST_CASE(Http_Make, HttpFixture)
 {
     TestStream::AddResponse("HTTP/1.1 200 OK\r\nAccept-Ranges: bytes\r\nContent-Length: 7\r\n");
@@ -110,16 +113,33 @@ FIXTURE_TEST_CASE(Http_Read, HttpFixture)
     REQUIRE_EQ( string ( "content" ), string ( buf, num_read ) );
 }
 
+#ifndef WINDOWS
+//TODO: figure out certificate validation on Windows
 FIXTURE_TEST_CASE(VDB_3661, HttpFixture)
 {
+    KEndPoint ep;
+#define PUBLIC_DOCS "public_docs.crg.es"
+    String dns;
+    CONST_STRING(&dns, PUBLIC_DOCS);
+    rc_t rc = KNSManagerInitDNSEndpoint(m_mgr, &ep, &dns, 80);
+    if (rc ==
+        SILENT_RC(rcNS, rcNoTarg, rcValidating, rcConnection, rcNotFound))
+    {
+        cerr << "unable to resolve host address " << PUBLIC_DOCS <<
+            ". Skipping VDB_3661 test..." << endl;
+        return;
+    }
+    REQUIRE_RC(rc);
+
     const KFile * file = NULL;
     REQUIRE_RC ( KNSManagerMakeHttpFile ( m_mgr, & file, NULL, 0x01010000,
-       "http://public_docs.crg.es/rguigo/Papers/2017_lagarde-uszczynska_CLS/data/trackHub//dataFiles/hsAll_Cap1_Brain_hiSeq.bam"
+       "http://" PUBLIC_DOCS "/rguigo/Papers/2017_lagarde-uszczynska_CLS"
+        "/data/trackHub//dataFiles/hsAll_Cap1_Brain_hiSeq.bam"
       ) );
 
     uint64_t size = 0;
     REQUIRE_RC ( KFileSize ( file, & size ) );
-    size = std::min(size, decltype(size)(4 * 1024 * 1024));
+    size = min(size, decltype(size)(4 * 1024 * 1024));
 
     void * buffer = malloc ( size );
     REQUIRE_NOT_NULL ( buffer );
@@ -135,6 +155,8 @@ FIXTURE_TEST_CASE(VDB_3661, HttpFixture)
 
     REQUIRE_RC ( KFileRelease ( file ) );
 }
+#endif
+#endif
 
 struct ReadThreadData
 {
@@ -146,6 +168,7 @@ struct ReadThreadData
     const char ** contents;
 };
 
+#ifdef ALL
 static rc_t CC read_thread_func( const KThread *self, void *data )
 {
     rc_t rc;
@@ -372,7 +395,7 @@ FIXTURE_TEST_CASE(HttpRetrySpecs_Construct_SleepsDoNotDecrease, HttpFixture)
     REQUIRE_RC ( KConfigRelease(kfg) );
     REQUIRE_RC ( HttpRetrySpecsDestroy ( & rs ) );
 }
-
+#endif
 
 //////////////////////////
 // HttpRetrier
@@ -411,6 +434,7 @@ public:
     KHttpRetrier m_retrier;
 };
 
+#ifdef ALL
 FIXTURE_TEST_CASE(HttpRetrier_Construct, RetrierFixture)
 {
     REQUIRE_RC ( KHttpRetrierInit ( & m_retrier, GetName(), m_mgr ) );
@@ -584,6 +608,7 @@ FIXTURE_TEST_CASE(HttpReliable_Read_Retry, HttpFixture)
     REQUIRE_EQ( string ( "content" ), string ( buf, num_read ) );
 }
 #endif
+#endif
 
 struct NV {
     String AcceptRanges;
@@ -600,6 +625,7 @@ struct NV {
 };
 static const NV s_v;
 
+#ifdef ALL
 TEST_CASE ( RepeatedHeader ) {
     rc_t rc = 0;
     KDirectory * dir = NULL;
@@ -737,11 +763,7 @@ TEST_CASE ( AllowAllCertificates )
     CONST_STRING ( & host, "www.google.com" );
 
     KClientHttp * https = NULL;
-#if WINDOWS
-    REQUIRE_RC_FAIL( KNSManagerMakeClientHttps( mgr, &https, NULL, 0x01010000, &host, 443 ) );
-#else
     REQUIRE_RC ( KNSManagerMakeClientHttps( mgr, &https, NULL, 0x01010000, &host, 443 ) );
-#endif
     RELEASE(KClientHttp, https);
 
     // tell manager to allow all certs
@@ -837,6 +859,28 @@ FIXTURE_TEST_CASE( KClientHttpResult_FormatMsg, HttpFixture)
     REQUIRE_EQ ( expected, string ((char*)buffer.base) );
     REQUIRE_RC ( KClientHttpResultRelease ( rslt ) );
 }
+#endif
+
+#ifdef ALL
+FIXTURE_TEST_CASE(GET_WITHOUT_CONTENT_LENGHT, HttpFixture) {
+    REQUIRE_RC(KNSManagerMakeHttpFile(m_mgr,
+        (const KFile **)&m_file, NULL, 0x01010000,
+        "http://ftp.ensembl.org/pub/data_files/homo_sapiens"
+        "/GRCh38/rnaseq/GRCh38.illumina.brain.1.bam.bai"));
+
+    uint64_t size;
+    REQUIRE_RC(KFileSize(m_file, &size));
+
+    char* buffer = new char[size];
+    REQUIRE(buffer);
+
+    size_t num_read = 0;
+    REQUIRE_RC(KFileRead(m_file, 0, buffer, size, &num_read));
+    REQUIRE_EQ((uint64_t)num_read, size);
+
+    delete[](buffer);
+}
+#endif
 
 //////////////////////////////////////////// Main
 
