@@ -64,7 +64,7 @@ class SRA_Fixture : public NGS_C_Fixture { };
 FIXTURE_TEST_CASE(SRA_ReadCollection_Open, SRA_Fixture)
 {
     ENTRY;
-    m_coll = NGS_ReadCollectionMake ( ctx, SRA_Accession );
+    m_coll = open ( ctx, SRA_Accession );
     REQUIRE ( ! FAILED () );
     REQUIRE_NOT_NULL ( m_coll );
     EXIT;
@@ -1066,8 +1066,9 @@ rc_t CC KMain ( int argc, char *argv [] )
     //assert(!KDbgSetString("KFG"));
     //assert(!KDbgSetString("VFS"));
     KConfigDisableUserSettings();
-    rc_t m_coll=NgsSraTestSuite(argc, argv);
-    return m_coll;
+    rc_t ret=NgsSraTestSuite(argc, argv);
+    NGS_C_Fixture::ReleaseCache();
+    return ret ;
 }
 
 }
